unit LockUnit;

interface

Uses
  Tomas;

Type
  TLockKeeper = class;
  TShip = class;
  TGenerator = class;

  TLockKeeper = class(TomasElement)
    Published
    Procedure Process; override;
    End;

  TShip = class(TomasElement)
    private
    ShipSide: Integer;
    ShipLength: Double;
    Published
    Constructor Create(Nm: String; L: Double; Side: Integer);
    End;

  TGenerator = class(TomasElement)
    public
    InterArrivalTimes: TExponentialDistribution;
    ShipLengths: TUniformDistribution;
    Published
    Constructor Create(Nm: String);
    Procedure Process; override;
    End;

   const
     Left = 0;
     Right = 1;


Var
  Generator: TGenerator;
  LockKeeper: TLockKeeper;
  Arrivals: Array[0..1] Of TomasQueue;
  Lock: TomasQueue;
  LockLength: Double;
  LockSide: Integer;

implementation

Procedure TLockKeeper.Process;
  Var
    Ship: TShip;
    NextShip: TShip;
    FreeLength: Double;
  Begin
  While TRUE Do
    Begin
    While (Arrivals[Left].Length + Arrivals[Right].Length = 0) Do
      Begin
      Standby;
      End;
    If Arrivals[LockSide].Length = 0 Then
      Begin
      Hold(10);
      LockSide :=  1 - LockSide;
      End;
    FreeLength:=LockLength;
    Ship:=Arrivals[LockSide].FirstElement;
    While (Ship<>Nil) And (FreeLength > 0) Do
      Begin
      NextShip:=Arrivals[LockSide].Successor(Ship);
      If Ship.ShipLength <= FreeLength Then
        Begin
        FreeLength:=FreeLength - Ship.ShipLength;
        Arrivals[LockSide].Remove(Ship);
        Hold(2);
        Lock.AddToTail(Ship);
        End;
      Ship:=NextShip;  
      End;
    Hold(10);
    LockSide :=  1 - LockSide;
    While Lock.Length > 0 Do
      Begin
      Ship:=Lock.FirstElement;
      Lock.Remove(Ship);
      Hold(2);
      Ship.Destroy;
      End;
    End;
  End;

Constructor TShip.Create(Nm: String; L:Double; Side: Integer);
  Begin
  inherited Create(Nm);
  ShipLength:=L;
  ShipSide:=Side;
  End;

Constructor TGenerator.Create(Nm: String);
  Begin
  inherited Create(Nm);
  InterArrivalTimes:=TExponentialDistribution.Create(520623,20);
  ShipLengths:=TUniformDistribution.Create(31659442,15,30);
  End;

Procedure TGenerator.Process;
  Var
    Seed: Integer;
    NewShip: TShip;
  Begin
  Seed:=20746501;
  While TRUE Do
    Begin
    NewShip:=TShip.Create('Ship',ShipLengths.Sample,Round(Rand(Seed)));
    Arrivals[NewShip.ShipSide].AddToTail(NewShip);
    Hold(InterArrivalTimes.Sample);
    End;
  End;

Procedure InitializeLock;
  Begin
  Lock:=TomasQueue.Create('Lock');
  Arrivals[0]:=TomasQueue.Create('LeftArrivals');
  Arrivals[1]:=TomasQueue.Create('RightArrivals');
  Generator:=TGenerator.Create('Generator');
  Generator.Start(TNow);
  LockSide:=Left;
  LockLength:=60;
  LockKeeper:=TLockKeeper.Create('LockKeeper');
  LockKeeper.Start(TNow);
  End;


Initialization
  InitializeLock;

end.
